@@ -1,5 +1,6 @@
 package RPC::Object::Common;
 use base qw(Exporter);
+use constant RELEASE_REF => 'rpc_object_release';
 use constant RESPONSE_ERROR => 'e';
 use constant RESPONSE_NORMAL => 'n';
 use constant WANT_LIST => 'l';
@@ -8,6 +9,7 @@ use strict;
 use warnings;
 
 our @EXPORT = qw(
+                 RELEASE_REF
                  RESPONSE_ERROR
                  RESPONSE_NORMAL
                  WANT_LIST
